"""
Copyright (c) 2014-present, aglean Inc.
"""
import pytest
from django.conf import settings


def pytest_configure():
    settings.configure(
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'anorganization'
        ],
        DATABASES={'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory',
            }
        }
    )



"""
Copyright (c) 2014-present, aglean Inc.
"""
import pytest
from django.contrib.auth import get_user_model

from anorganization.models import Organization, Membership
from anorganization.resolvers import resolve_anorganizations, \
        resolve_anorganization_memberships


@pytest.fixture(scope='module')
def django_db_setup(django_db_setup, django_db_blocker):
    with django_db_blocker.unblock():
        user_1 = get_user_model().objects.create_user('John')
        user_2 = get_user_model().objects.create_user('Marry')

        org_1 = Organization.objects.create(name='red')
        org_2 = Organization.objects.create(name='black')
        org_3 = Organization.objects.create(name='white')

        member_1 = Membership.objects.create(user=user_1,
                                             organization=org_1)
        member_2 = Membership.objects.create(user=user_1,
                                             organization=org_1)
        member_3 = Membership.objects.create(user=user_2,
                                             organization=org_2,
                                             is_valid=False)


class TestOrganization:
    @pytest.mark.django_db
    def test_resolve_organizations_from_root_without_args(self):
        assert resolve_anorganizations(None, None).count() == 3

    @pytest.mark.django_db
    def test_resolve_organizations_from_root_with_args(self):
        assert resolve_anorganizations(None, None, name='ack').count() == 1

    @pytest.mark.django_db
    def test_resolve_organizations_from_obj_without_args(self):
        queryset = Organization.objects.all()
        assert resolve_anorganizations(queryset, None).count() == 3

    @pytest.mark.django_db
    def test_resolve_organizations_from_obj_with_args(self):
        queryset = Organization.objects.all()
        assert resolve_anorganizations(queryset, None, name='ack').count() == 1


class TestMembership:
    @pytest.mark.django_db
    def test_resolve_memberships_from_root_without_args(self):
        assert resolve_anorganization_memberships(None, None).count() == 3

    @pytest.mark.django_db
    def test_resolve_memberships_from_root_with_args(self):
        assert resolve_anorganization_memberships(None, None,
                                                  is_valid=False).count() == 1

    @pytest.mark.django_db
    def test_resolve_memberships_from_obj_without_args(self):
        queryset = Membership.objects.all()
        assert resolve_anorganizations(queryset, None).count() == 3

    @pytest.mark.django_db
    def test_resolve_memberships_from_obj_with_args(self):
        queryset = Membership.objects.all()
        assert resolve_anorganization_memberships(queryset,
                                                  None,
                                                  is_valid=False).count() == 1



"""
Copyright (c) 2014-present, aglean Inc.
"""
from pathlib import PurePath
from secrets import token_urlsafe

from django.conf import settings
from django.utils import timezone


def image_path(instance, filename):
    fixed_filename = PurePath(filename).stem[:8] \
            if len(PurePath(filename).stem) > 8 else PurePath(filename).stem

    cleaned_filename = (
        f"{timezone.now().strftime('%Y%m%d%H%M%S%f')}-"
        f'{fixed_filename}{PurePath(filename).suffix}'
    )

    if getattr(settings, 'ANORGANIZATION_USE_TOKEN_FILENAME', False):
        return PurePath(instance._meta.app_label).joinpath(
                f'{token_urlsafe()}{PurePath(filename).suffix}')
    else:
        return PurePath(instance._meta.app_label).joinpath(cleaned_filename)


def image_url(instance):
    url = ''
    if instance.image:
        url = instance.image.url

    return url



"""
Copyright (c) 2014-present, aglean Inc.
"""


class OrganizationAdminMixin():
    list_display = ('id', 'name', 'dn', 'image', 'image_url',
                    'valid_member_count', 'invalid_member_count')
    search_fields = ('name',)


class MembershipAdminMixin():
    list_display = ('id', 'user', 'organization', 'serial_number', 'dn',
                    'is_valid', 'created_at', 'updated_at')
    list_filter = ('organization__name', 'is_valid')
    search_fields = ('user__username', 'organization__name')
    autocomplete_fields = ('user', 'organization')



# Generated by Django 4.1.6 on 2023-02-17 08:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import anorganization.utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('serial_number', models.CharField(blank=True, max_length=100)),
                ('dn', models.CharField(blank=True, max_length=255)),
                ('attributes', models.JSONField(blank=True)),
                ('image', models.ImageField(blank=True, help_text='Upload file should under size limitation, with png, jpg or jpeg file extensions.', upload_to=anorganization.utils.image_path)),
                ('is_valid', models.BooleanField(default=True, help_text='Designates whether the membership is valid.')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['-updated_at'],
            },
        ),
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('dn', models.CharField(blank=True, max_length=255)),
                ('attributes', models.JSONField(blank=True)),
                ('image', models.ImageField(blank=True, help_text='Upload file should under size limitation, with png, jpg or jpeg file extensions.', upload_to=anorganization.utils.image_path)),
                ('members', models.ManyToManyField(through='anorganization.Membership', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='membership',
            name='organization',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='anorganization.organization'),
        ),
        migrations.AddField(
            model_name='membership',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]
